
TOOL.Category		= "Physics"
TOOL.Name			= "#Water Tool"
TOOL.Command		= nil
TOOL.ConfigName		= ""

TOOL.ClientConVar[ "force" ] = "1500"
TOOL.ClientConVar[ "viscosity" ] = "1"
TOOL.ClientConVar[ "r" ] = "255"
TOOL.ClientConVar[ "g" ] = "255"
TOOL.ClientConVar[ "b" ] = "255"
TOOL.ClientConVar[ "a" ] = "255"
TOOL.ClientConVar[ "model" ] = "models/items/combine_rifle_ammo01.mdl"
TOOL.ClientConVar[ "key" ] = "0"
TOOL.ClientConVar[ "toggle" ] = "0"
TOOL.ClientConVar[ "collide" ] = "0"

cleanup.Register( "wateremitter" )

if CLIENT then
	language.Add("Tool_water_tool_name", "Water STool")
	language.Add("Tool_water_tool_desc", "Creates water emitters with physical behavior.")
	language.Add("Tool_water_tool_0", "Left Click to spawn a water emitter. Reload to delete all of your spawned water.")
	language.Add("Cleanup_wateremitter","Water Emitter")
	language.Add("Cleaned_wateremitter", "Cleaned up water emitter")
	language.Add("Undone_wateremitter", "Water emitter undone")
end

function TOOL:LeftClick( trace )

	if trace.Entity && trace.Entity:IsPlayer() then return false end
	
	// -- -- If there's no physics object then we can't constraint it!
	if ( SERVER && !util.IsValidPhysicsObject( trace.Entity, trace.PhysicsBone ) ) then return false end
	
	if (CLIENT) then return true end
	
	local ply = self:GetOwner()
	
	local force			= self:GetClientNumber( "force" )
	local viscosity		= self:GetClientNumber( "viscosity" )
	local r				= self:GetClientNumber( "r" )
	local g				= self:GetClientNumber( "g" )
	local b				= self:GetClientNumber( "b" )
	local a				= self:GetClientNumber( "a" )
	local model			= self:GetClientInfo( "model" )
	local key 			= self:GetClientNumber( "key" ) 
	local toggle		= self:GetClientNumber( "toggle" ) 
	local collide		= self:GetClientNumber( "collide" ) 
	
	// -- -- If we shot a Water Entity change its data
	if ( trace.Entity:IsValid() && trace.Entity:GetClass() == "water_emitter" && trace.Entity:GetTable().pl == ply ) then

		trace.Entity:GetTable():SetForce( force )
		trace.Entity:GetTable():SetViscosity( viscosity )
		if (collide == 0) then
			trace.Entity:SetCollisionGroup( 4 )
		end
		trace.Entity:GetTable():SetColor( r, g, b, a )
		trace.Entity:GetTable():SetToggle( toggle == 1 )
		trace.Entity:GetTable():SetCollide( collide == 1 )

		trace.Entity:GetTable().force		= force
		trace.Entity:GetTable().viscosity	= viscosity
		trace.Entity:GetTable().toggle		= toggle

		return true
	end
	
	if ( !self:GetSWEP():CheckLimit( "thrusters" ) ) then return false end

	if (not util.IsValidModel(model)) then return false end
	if (not util.IsValidProp(model)) then return false end		// -- Allow ragdolls to be used?

	local Ang = trace.HitNormal:Angle()
	Ang.pitch = Ang.pitch + 90

	local waterEnt = MakeWaterEmitter( ply, model, Ang, trace.HitPos, key, force, viscosity, r, g, b, a, toggle, collide )
	
	local min = waterEnt:OBBMins()
	waterEnt:SetPos( trace.HitPos - trace.HitNormal * min.z )
	
	local const, nocollide
	
	// -- Don't weld to world
	if ( trace.Entity:IsValid() ) then
	
		const = constraint.Weld( waterEnt, trace.Entity, 0, trace.PhysicsBone, 0 )
		trace.Entity:DeleteOnRemove( waterEnt )
		waterEnt:GetPhysicsObject():EnableCollisions( false )
		waterEnt:GetTable().nocollide = true
		
	end
	
	undo.Create("wateremitter")
		undo.AddEntity( waterEnt )
		undo.AddEntity( const )
		undo.SetPlayer( ply )
	undo.Finish()
		
	ply:AddCleanup( "wateremitter", waterEnt )
	ply:AddCleanup( "wateremitter", const )
	
	return true

end

function TOOL:Reload()
	for k,v in pairs(ents.FindByClass("water_projectile")) do
		//if v:GetNetworkedInt("player") == self.Owner then
			v:Remove()
		//end
	end
end

if (SERVER) then

	function MakeWaterEmitter( pl, Model, Ang, Pos, key, force, viscosity, r, g, b, a, toggle, collide, Vel, aVel, frozen )
	
		// -- --if ( !pl:CheckLimit( "wateremitter" ) ) then return false end
	
		local waterEnt = ents.Create( "water_emitter" )
	
		if (!waterEnt:IsValid()) then return false end
		waterEnt:SetModel( Model )

		waterEnt:SetAngles( Ang )
		waterEnt:SetPos( Pos )
		waterEnt:Spawn()

		waterEnt:GetTable():SetForce( force )
		waterEnt:GetTable():SetViscosity( viscosity )
		waterEnt:GetTable():SetColor( r, g, b, a )
		waterEnt:GetTable():SetToggle( toggle == 1 )
		waterEnt:GetTable():SetCollide( collide == 1 )
		waterEnt:GetTable():SetPlayer( pl )

		numpad.OnDown( 	 pl, 	key, 	"Wind_On", 		waterEnt, 1 )
		numpad.OnUp( 	 pl, 	key, 	"Wind_Off", 	waterEnt, 1 )

		local ttable = {
			key	= key,
			force	= force,
			toggle	= toggle,
			pl	= pl,
			}

		table.Merge(waterEnt:GetTable(), ttable )
		
		pl:AddCount( "wateremitter", waterEnt )

		return waterEnt
		
	end
	
	duplicator.RegisterEntityClass( "water_emitter", MakeWaterEmitter, "Model", "Ang", "Pos", "key", "force", "toggle", "Vel", "aVel", "frozen" )

end

function TOOL:UpdateGhostWater( ent, player )

	if ( !ent ) then return end
	if ( !ent:IsValid() ) then return end

	local tr 	= utilx.GetPlayerTrace( player, player:GetCursorAimVector() )
	local trace 	= util.TraceLine( tr )
	if (!trace.Hit) then return end
	
	if (trace.Entity && trace.Entity:GetClass() == "water_emitter" || trace.Entity:IsPlayer()) then
	
		ent:SetNoDraw( true )
		return
		
	end
	
	local Ang = trace.HitNormal:Angle()
	Ang.pitch = Ang.pitch + 90
	
	local min = ent:OBBMins()
	 ent:SetPos( trace.HitPos - trace.HitNormal * min.z )
	ent:SetAngles( Ang )
	
	ent:SetNoDraw( false )
	
end

function TOOL:Think()

	if (!self.GhostEntity || !self.GhostEntity:IsValid() || self.GhostEntity:GetModel() != self:GetClientInfo( "model" )) then
		self:MakeGhostEntity( self:GetClientInfo( "model" ), Vector(0,0,0), Angle(0,0,0) )
	end
	
	self:UpdateGhostWater( self.GhostEntity, self:GetOwner() )
	
end


function TOOL.BuildCPanel( CPanel )
	
	CPanel:AddControl("Header", {
		Text = "#Tool_water_tool_name",
		Description = "#Tool_water_tool_desc"
		})
		
	CPanel:AddControl("Numpad", { Label = "#Activate Emitter", Command = "water_tool_key", ButtonSize = 22 } )
	CPanel:AddControl("Slider",{Label = "#Force",	Type = "Int", Min = 10, Max = 50000, Command = "water_tool_force"})
	CPanel:AddControl("Slider",{Label = "#Viscosity",	Type = "Int", Min = 0, Max = 50, Command = "water_tool_viscosity"})
	CPanel:AddControl("Checkbox", { Label = "#Toggle", Description = "#Toggle", Command = "water_tool_toggle" })
	CPanel:AddControl("Checkbox", { Label = "#Collides", Description = "#Collides", Command = "water_tool_collide" })
	CPanel:AddControl("Color", { Label = "#Color",
					Red	= "water_tool_r",
					Green	= "water_tool_g",
					Blue	= "water_tool_b",
					Alpha	= "water_tool_a",
					Multiplier = "255" } )
end
